@@ -1045,6 +1045,38 @@ struct connman_device *__connman_device_find_device(
 	return NULL;
 }
 
+/**
+ * connman_device_set_regdom
+ * @device: device structure
+ * @alpha2: string representing regulatory domain
+ *
+ * Set regulatory domain on device basis
+ */
+int connman_device_set_regdom(struct connman_device *device,
+						const char *alpha2)
+{
+	if (device->driver == NULL || device->driver->set_regdom == NULL)
+		return -ENOTSUP;
+
+	if (device->powered == FALSE)
+		return -EINVAL;
+
+	return device->driver->set_regdom(device, alpha2);
+}
+
+/**
+ * connman_device_regdom_notify
+ * @device: device structure
+ * @alpha2: string representing regulatory domain
+ *
+ * Notify on setting regulatory domain on device basis
+ */
+void connman_device_regdom_notify(struct connman_device *device,
+					int result, const char *alpha2)
+{
+	__connman_technology_notify_regdom_by_device(device, result, alpha2);
+}
+
 int __connman_device_request_scan(enum connman_service_type type)
 {
 	connman_bool_t success = FALSE;
@@ -1157,6 +1189,54 @@ list:
 	return FALSE;
 }
 
+static void cleanup_devices(void)
+{
+	/*
+	 * Check what interfaces are currently up and if connman is
+	 * suppose to handle the interface, then cleanup the mess
+	 * related to that interface. There might be weird routes etc
+	 * that are related to that interface and that might confuse
+	 * connmand. So in this case we just turn the interface down
+	 * so that kernel removes routes/addresses automatically and
+	 * then proceed the startup.
+	 *
+	 * Note that this cleanup must be done before rtnl/detect code
+	 * has activated interface watches.
+	 */
+
+	char **interfaces;
+	int i;
+
+	interfaces = __connman_inet_get_running_interfaces();
+
+	if (interfaces == NULL)
+		return;
+
+	for (i = 0; interfaces[i] != NULL; i++) {
+		connman_bool_t filtered;
+		int index;
+
+		filtered = __connman_device_isfiltered(interfaces[i]);
+		if (filtered == TRUE)
+			continue;
+
+		index = connman_inet_ifindex(interfaces[i]);
+		if (index < 0)
+			continue;
+
+		DBG("cleaning up %s index %d", interfaces[i], index);
+
+		connman_inet_ifdown(index);
+
+		/*
+		 * ConnMan will turn the interface UP automatically so
+		 * no need to do it here.
+		 */
+	}
+
+	g_strfreev(interfaces);
+}
+
 int __connman_device_init(const char *device, const char *nodevice)
 {
 	DBG("");
@@ -1167,6 +1247,8 @@ int __connman_device_init(const char *device, const char *nodevice)
 	if (nodevice != NULL)
 		nodevice_filter = g_strsplit(nodevice, ",", -1);
 
+	cleanup_devices();
+
 	return 0;
 }
 
